# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Inherit account_invoice_line to add header discount and early payment discount treatment"""

from osv import osv, fields
from tools.translate import _
import decimal_precision as dp

class account_invoice_line(osv.osv):

    _inherit = "account.invoice.line"

    def line_apply_header_discounts(self, cr, uid, ids, context=None):
        delivery_carrier_obj = self.pool.get('delivery.carrier')
        line = self.browse(cr,uid,ids)
        if not line[0]:
            return True

        ids = delivery_carrier_obj.search(cr, uid, [('active','=',True)])
        for carrier in delivery_carrier_obj.browse(cr,uid,ids, context=context):
            if carrier.product_id.id == line[0].product_id.id:
                return False
        return True

    def compute_discount(self, base, discount):
        if not discount:
            return 0.0
        else:
            return base * (discount or 0.0)/100.0

    def _discount_amount(self, cr, uid, ids, field_name, arg,context):
        res={}
        for line in self.browse(cr,uid,ids):
            res[line.id] = self.compute_discount(line.price_unit*line.quantity,line.discount)
        return res

    def _discount_extra_amount(self, cr, uid, ids, field_name, arg,context):
        res={}
        cur_obj = self.pool.get('res.currency')
        for line in self.browse(cr,uid,ids):
            d1 = line.invoice_id.header_discount
            d2 = line.invoice_id.early_payment_discount
            damount1 = 0.0
            damount2 = 0.0
            if line.line_apply_header_discounts():
                if d1:
                    #damount1 = self.compute_discount(line.price_unit*line.quantity - line.discount_amount, d1)
                    damount1 = self.compute_discount(line.price_subtotal, d1)
                    damount1 = cur_obj.round(cr, uid, line.invoice_id.currency_id, damount1)
                if d2:
                    #damount2 = self.compute_discount(line.price_unit*line.quantity - line.discount_amount - damount1, d2)
                    damount2 = self.compute_discount(line.price_subtotal - damount1, d2)
                    damount2 = cur_obj.round(cr, uid, line.invoice_id.currency_id, damount2)

            res[line.id] = damount1 + damount2

        return res

    def _amount_line_discount_extra(self, cr, uid, ids, field_name, arg,context):
        res = {}
        for line in self.browse(cr, uid, ids):
            res[line.id] = line.price_subtotal - line.discount_extra_amount
            #res[line.id] = line.price_unit * line.quantity - line.discount_amount - line.discount_extra_amount

        return res

    def _price_unit_discount(self, cr, uid, ids, field_name, arg,context):
        res = {}
        for line in self.browse(cr, uid, ids):
            if not line.quantity:
                res[line.id]=0.0
            else:
                res[line.id] = line.price_subtotal_extra/line.quantity

        return res

    _columns = {
        'discount_amount': fields.function(_discount_amount,method=True,string='Discount Amount'),
        'discount_extra_amount': fields.function(_discount_extra_amount,method=True,string='Additional Discount Amount'),
        'price_subtotal_extra': fields.function(_amount_line_discount_extra, method=True, string='Subtotal'),
        'price_unit_discount': fields.function(_price_unit_discount,method=True,string='Discounted Unit Price (header and early payment discounts'),
    }

    def _get_price_subtotal_accounting(self, cr, uid, line, context=None):
        cur_obj = self.pool.get('res.currency')
        #amount = line.price_unit * line.quantity - line.discount_amount
        amount = line.price_subtotal
        amount = cur_obj.round(cr, uid, line.invoice_id.currency_id, amount)
        if self.line_apply_header_discounts(cr, uid, line, context=context):
            #damount1 = self.compute_discount(line.price_unit*line.quantity - line.discount_amount, line.invoice_id.header_discount)
            damount1 = self.compute_discount(line.price_subtotal, line.invoice_id.header_discount)
            damount1 = cur_obj.round(cr, uid, line.invoice_id.currency_id, damount1)
            amount = amount - damount1
        return amount

    def _get_price_unit_accounting(self, cr, uid, line, context=None):
        amount = self._get_price_subtotal_accounting(cr, uid, line, context=context)
        if not line.quantity:
            return 0.0
        else:
            return amount/line.quantity

    def move_line_get_item(self, cr, uid, line, context=None):
        return {
            'type':'src',
            'name': line.name[:64],
            'price_unit':self._get_price_unit_accounting(cr, uid, line, context=context),
            'quantity':line.quantity,
            'price': self._get_price_subtotal_accounting(cr, uid, line, context=context),
            'account_id':line.account_id.id,
            'product_id':line.product_id.id,
            'uos_id':line.uos_id.id,
            'account_analytic_id':line.account_analytic_id.id,
            'taxes':line.invoice_line_tax_id,
        }

account_invoice_line()
