# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Inherit sale_order to add header discount and early payment discount"""

from osv import fields, osv
import decimal_precision as dp
import time

class sale_order(osv.osv):
	"""Inherit sale_order to add header discount and early payment discount"""

	_inherit ="sale.order"
	
	def line_apply_header_discounts(self, cr, uid, line, context=None):
		return True
		
	def _get_order(self, cr, uid, ids, context=None):
		result = {}
		for line in self.pool.get('sale.order.line').browse(cr, uid, ids, context=context):
			result[line.order_id.id] = True
		return result.keys()
		
	def _amount_line_tax(self, cr, uid, line, context=None):
		val = 0.0
		amount_line = 0.0
		if self.line_apply_header_discounts(cr, uid, line, context=context):
			amount_line = line.price_unit * (1-(line.discount or 0.0)/100.0) * (1-(line.order_id.header_discount or 0.0)/100.0) * (1-(line.order_id.early_payment_discount or 0.0)/100.0)
		else:
			amount_line = line.price_unit * (1-(line.discount or 0.0)/100.0)
		for c in self.pool.get('account.tax').compute_all(cr, uid, line.tax_id, amount_line, line.product_uom_qty, line.order_id.partner_invoice_id.id, line.product_id, line.order_id.partner_id)['taxes']:
			val += c.get('amount', 0.0)
		return val
		
	def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
		cur_obj = self.pool.get('res.currency')
		res = {}
		for order in self.browse(cr, uid, ids, context=context):
			res[order.id] = {
				'amount_untaxed': 0.0,
				'amount_tax': 0.0,
				'amount_total': 0.0,
				'gross_amount': 0.0,
				'header_discount_amount': 0.0,
				'early_discount_amount': 0.0,
			}
			val = val1 = val2 = val3 = val4 = 0.0
			cur = order.pricelist_id.currency_id
			for line in order.order_line:
				val1 += line.price_subtotal
				val += self._amount_line_tax(cr, uid, line, context=context)
				if self.line_apply_header_discounts(cr, uid, line, context=context):
					val2 += cur_obj.round(cr, uid, cur, line.price_subtotal * (1-(order.header_discount or 0.0)/100.0) * (1-(order.early_payment_discount or 0.0)/100.0))
					val3 += cur_obj.round(cr, uid, cur, (line.price_subtotal  * (order.header_discount or 0.0)/100.0))
					val4 += cur_obj.round(cr, uid, cur, (line.price_subtotal * (1-(order.header_discount or 0.0)/100.0)  * (order.early_payment_discount or 0.0)/100.0))
				else:
					val2 += line.price_subtotal
			
			res[order.id]['amount_tax'] = cur_obj.round(cr, uid, cur, val)
			res[order.id]['amount_untaxed'] = cur_obj.round(cr, uid, cur, val2)
			res[order.id]['gross_amount'] = cur_obj.round(cr, uid, cur, val1)
			res[order.id]['header_discount_amount'] = cur_obj.round(cr, uid, cur, val3)
			res[order.id]['early_discount_amount'] = cur_obj.round(cr, uid, cur, val4)
			res[order.id]['amount_total'] = res[order.id]['amount_untaxed'] + res[order.id]['amount_tax']
		return res
		
	_columns = {
		'header_discount': fields.float('Disc.(%)', digits=(16,2), help="Header discount from costumer"),
		'early_payment_discount': fields.float('E.P. disc.(%)', digits=(16,2), help="Early payment discount"),
		'header_discount_amount': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Header Discount Amount',
			store = {
				'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'early_payment_discount', 'header_discount'], 10),
				'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
			},
			multi='sums', help="The amount without tax and header discounts."),
		'early_discount_amount': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Early payment Discount Amount',
			store = {
				'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'early_payment_discount', 'header_discount'], 10),
				'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
			},
			multi='sums', help="The amount without tax and header discounts."),
		'gross_amount': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Gross Amount',
			store = {
				'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'early_payment_discount', 'header_discount'], 10),
				'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
			},
			multi='sums', help="The amount without tax and header discounts."),
		'amount_untaxed': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Untaxed Amount',
			store = {
				'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'early_payment_discount', 'header_discount'], 10),
				'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
			},
			multi='sums', help="The amount without tax."),
		'amount_tax': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Taxes',
			store = {
				'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'early_payment_discount', 'header_discount'], 10),
				'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
			},
			multi='sums', help="The tax amount."),
		'amount_total': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Total',
			store = {
				'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'early_payment_discount', 'header_discount'], 10),
				'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
			},
			multi='sums', help="The total amount."),
	}

	def onchange_partner_id(self, cr, uid, ids, part):
		"""extend this event for delete early payment discount if it isn't valid to new partner or add new early payment discount"""
		res = super(sale_order, self).onchange_partner_id(cr, uid, ids, part)
		#res = self.onchange_partner_id(cr, uid, ids, part)
		if not part:
			res['value']['early_payment_discount'] = False
			res['value']['header_discount'] = False
			return res
		else:
			partner = self.pool.get('res.partner').browse(cr, uid, part, context=None)
			res['value']['header_discount'] = partner.header_discount
			res['value']['early_payment_discount'] = False
			if partner.apply_early_discount:
				if res['value'].get('payment_term', False):
					pt_obj = self.pool.get('account.payment.term')
					pt = pt_obj.browse(cr, uid, res['value']['payment_term'], context=None) 
					if pt and pt.apply_early_discount:
						if partner.early_payment_discount > 0:
							res['value']['early_payment_discount'] = partner.early_payment_discount
						else:
							res['value']['early_payment_discount'] = pt.early_payment_discount
			
		return res
		
	# def onchange_partner_id2(self, cr, uid, ids, part, header_discount=False, early_payment_discount=False, payment_term=False):
		# """extend this event for delete early payment discount if it isn't valid to new partner or add new early payment discount"""
		# res = self.onchange_partner_id(cr, uid, ids, part)
		# if not part:
			# res['value']['early_payment_discount'] = False
			# res['value']['header_discount'] = False
			# return res
		# else:
			# partner = self.pool.get('res.partner').browse(cr, uid, part, context=None)
			# res['value']['header_discount'] = partner.header_discount
			# res['value']['early_payment_discount'] = False
			# if partner.apply_early_discount:
				# if res['value'].get('payment_term', False):
					# pt_obj = self.pool.get('account.payment.term')
					# pt = pt_obj.browse(cr, uid, res['value']['payment_term'], context=None) 
					# if pt and pt.apply_early_discount:
						# if partner.early_payment_discount > 0:
							# res['value']['early_payment_discount'] = partner.early_payment_discount
						# else:
							# res['value']['early_payment_discount'] = pt.early_payment_discount
			
		# return res

	def onchange_payment_term(self, cr, uid, ids, payment_term, part=False):
		"""onchange event to update early payment dicount when the payment term changes"""
		res = {}
		if not payment_term or not part:
			res['early_payment_discount'] = False
			return {'value': res}
		else:
			partner = self.pool.get('res.partner').browse(cr, uid, part, context=None)
			payment_term_obj = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=None)
			if partner.apply_early_discount and payment_term_obj.apply_early_discount:
				if partner.early_payment_discount > 0:
					res['early_payment_discount'] = partner.early_payment_discount
				else:
					res['early_payment_discount'] = payment_term_obj.early_payment_discount
		
		return {'value': res}


	def _make_invoice(self, cr, uid, order, lines, context=None):
		journal_obj = self.pool.get('account.journal')
		inv_obj = self.pool.get('account.invoice')
		obj_invoice_line = self.pool.get('account.invoice.line')
		if context is None:
			context = {}

		journal_ids = journal_obj.search(cr, uid, [('type', '=', 'sale'), ('company_id', '=', order.company_id.id)], limit=1)
		if not journal_ids:
			raise osv.except_osv(_('Error !'),
				_('There is no sales journal defined for this company: "%s" (id:%d)') % (order.company_id.name, order.company_id.id))
		a = order.partner_id.property_account_receivable.id
		pay_term = order.payment_term and order.payment_term.id or False
		invoiced_sale_line_ids = self.pool.get('sale.order.line').search(cr, uid, [('order_id', '=', order.id), ('invoiced', '=', True)], context=context)
		from_line_invoice_ids = []
		for invoiced_sale_line_id in self.pool.get('sale.order.line').browse(cr, uid, invoiced_sale_line_ids, context=context):
			for invoice_line_id in invoiced_sale_line_id.invoice_lines:
				if invoice_line_id.invoice_id.id not in from_line_invoice_ids:
					from_line_invoice_ids.append(invoice_line_id.invoice_id.id)
		for preinv in order.invoice_ids:
			if preinv.state not in ('cancel',) and preinv.id not in from_line_invoice_ids:
				for preline in preinv.invoice_line:
					inv_line_id = obj_invoice_line.copy(cr, uid, preline.id, {'invoice_id': False, 'price_unit': -preline.price_unit})
					lines.append(inv_line_id)
		inv = {
			'name': order.client_order_ref or '',
			'origin': order.name,
			'type': 'out_invoice',
			'reference': order.client_order_ref or order.name,
			'account_id': a,
			'partner_id': order.partner_id.id,
			'journal_id': journal_ids[0],
			'address_invoice_id': order.partner_invoice_id.id,
			'address_contact_id': order.partner_order_id.id,
			'invoice_line': [(6, 0, lines)],
			'currency_id': order.pricelist_id.currency_id.id,
			'comment': order.note,
			'payment_term': pay_term,
			'fiscal_position': order.fiscal_position.id or order.partner_id.property_account_position.id,
			'date_invoice': context.get('date_invoice',False),
			'company_id': order.company_id.id,
			'user_id': order.user_id and order.user_id.id or False,
			'early_payment_discount': order.early_payment_discount,
			'header_discount': order.header_discount,
		}
		inv.update(self._inv_get(cr, uid, order))
		inv_id = inv_obj.create(cr, uid, inv, context=context)
		data = inv_obj.onchange_payment_term_date_invoice(cr, uid, [inv_id], pay_term, time.strftime('%Y-%m-%d'))
		if data.get('value', False):
			inv_obj.write(cr, uid, [inv_id], data['value'], context=context)
		inv_obj.button_compute(cr, uid, [inv_id])
		return inv_id

sale_order()

# class sale_order_line(osv.osv):
	# """Inherit sale_order to add header discount and early payment discount"""

	# _inherit ="sale.order.line"
	
	# def line_apply_header_discounts(self, cr, uid, line, context=None):
		# return True
		
# sale_order_line()