# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Inherit stock_picking to add header discount and early payment discount from sale order"""

from osv import osv

class stock_picking(osv.osv):
	"""Inherit stock_picking to add header discount and early payment discount from sale order"""

	_inherit = "stock.picking"

	def _get_header_discount(self,cr,uid,pick):
		if pick.sale_id:
			return pick.sale_id.header_discount
		else:
			return False
	
	def _get_early_payment_discount(self,cr,uid,pick):
		if pick.sale_id:
			return pick.sale_id.early_payment_discount
		else:
			return False
	
	def action_invoice_create(self, cr, uid, ids, journal_id=False,
			group=False, type='out_invoice', context=None):
		
		invoice_obj = self.pool.get('account.invoice')
		picking_obj = self.pool.get('stock.picking')
		res = super(stock_picking, self).action_invoice_create(cr, uid, ids, journal_id=journal_id,
			group=group, type=type, context=context)
		
		picking_ids = res.keys()
		for picking in picking_obj.browse(cr, uid, picking_ids, context=context):
			if not picking.sale_id:
				continue
			discount_header=self._get_header_discount(cr,uid,picking)
			discount_early_payment=self._get_early_payment_discount(cr,uid,picking)
			invoice_obj.write(cr, uid, [res[picking.id]], {'early_payment_discount': discount_early_payment, 'header_discount': discount_header}, context=context)
			invoice_obj.button_compute(cr, uid, [res[picking.id]], context=context,
					set_total=(type in ('in_invoice', 'in_refund')))
					
		return res
		
	# def action_invoice_create(self, cr, uid, ids, journal_id=False,
			# group=False, type='out_invoice', context=None):
		# """ Creates invoice based on the invoice state selected for picking.
		# @param journal_id: Id of journal
		# @param group: Whether to create a group invoice or not
		# @param type: Type invoice to be created
		# @return: Ids of created invoices for the pickings
		# """
		# if context is None:
			# context = {}

		# invoice_obj = self.pool.get('account.invoice')
		# invoice_line_obj = self.pool.get('account.invoice.line')
		# address_obj = self.pool.get('res.partner.address')
		# invoices_group = {}
		# res = {}
		# inv_type = type
		
		# for picking in self.browse(cr, uid, ids, context=context):
			# if picking.invoice_state != '2binvoiced':
				# continue
			# payment_term_id = False
			# partner =  picking.address_id and picking.address_id.partner_id
			# if not partner:
				# raise osv.except_osv(_('Error, no partner !'),
					# _('Please put a partner on the picking list if you want to generate invoice.'))

			# if not inv_type:
				# inv_type = self._get_invoice_type(picking)

			# if inv_type in ('out_invoice', 'out_refund'):
				# account_id = partner.property_account_receivable.id
				# payment_term_id = self._get_payment_term(cr, uid, picking)
			# else:
				# account_id = partner.property_account_payable.id
				
			# discount_header=self._get_header_discount(cr,uid,picking)
			# discount_early_payment=self._get_early_payment_discount(cr,uid,picking)
			
			# address_contact_id, address_invoice_id = \
					# self._get_address_invoice(cr, uid, picking).values()
			# address = address_obj.browse(cr, uid, address_contact_id, context=context)

			# comment = self._get_comment_invoice(cr, uid, picking)
			# if group and partner.id in invoices_group:
				# invoice_id = invoices_group[partner.id]
				# invoice = invoice_obj.browse(cr, uid, invoice_id)
				# invoice_vals = {
					# 'name': (invoice.name or '') + ', ' + (picking.name or ''),
					# 'origin': (invoice.origin or '') + ', ' + (picking.name or '') + (picking.origin and (':' + picking.origin) or ''),
					# 'comment': (comment and (invoice.comment and invoice.comment+"\n"+comment or comment)) or (invoice.comment and invoice.comment or ''),
					# 'date_invoice':context.get('date_inv',False),
					# 'user_id':uid
				# }
				# invoice_obj.write(cr, uid, [invoice_id], invoice_vals, context=context)
			# else:
				# invoice_vals = {
					# 'name': picking.name,
					# 'origin': (picking.name or '') + (picking.origin and (':' + picking.origin) or ''),
					# 'type': inv_type,
					# 'account_id': account_id,
					# 'partner_id': address.partner_id.id,
					# 'address_invoice_id': address_invoice_id,
					# 'address_contact_id': address_contact_id,
					# 'comment': comment,
					# 'payment_term': payment_term_id,
					# 'fiscal_position': partner.property_account_position.id,
					# 'date_invoice': context.get('date_inv',False),
					# 'company_id': picking.company_id.id,
					# 'user_id':uid,
					# 'early_payment_discount': discount_early_payment,
					# 'header_discount': discount_header
				# }
				# cur_id = self.get_currency_id(cr, uid, picking)
				# if cur_id:
					# invoice_vals['currency_id'] = cur_id
				# if journal_id:
					# invoice_vals['journal_id'] = journal_id
				# else:
					# raise osv.except_osv(_('Invalid action !'), _('no hay diario!')) 
				# invoice_id = invoice_obj.create(cr, uid, invoice_vals,
						# context=context)
				# invoices_group[partner.id] = invoice_id
			# res[picking.id] = invoice_id
			# for move_line in picking.move_lines:
				# if move_line.state == 'cancel':
					# continue
				# origin = move_line.picking_id.name or ''
				# if move_line.picking_id.origin:
					# origin += ':' + move_line.picking_id.origin
				# if group:
					# name = (picking.name or '') + '-' + move_line.name
				# else:
					# name = move_line.name

				# if inv_type in ('out_invoice', 'out_refund'):
					# account_id = move_line.product_id.product_tmpl_id.\
							# property_account_income.id
					# if not account_id:
						# account_id = move_line.product_id.categ_id.\
								# property_account_income_categ.id
				# else:
					# account_id = move_line.product_id.product_tmpl_id.\
							# property_account_expense.id
					# if not account_id:
						# account_id = move_line.product_id.categ_id.\
								# property_account_expense_categ.id

				# price_unit = self._get_price_unit_invoice(cr, uid,
						# move_line, inv_type)
				# discount = self._get_discount_invoice(cr, uid, move_line)
				# tax_ids = self._get_taxes_invoice(cr, uid, move_line, inv_type)
				# account_analytic_id = self._get_account_analytic_invoice(cr, uid, picking, move_line)
				
				##set UoS if it's a sale and the picking doesn't have one
				# uos_id = move_line.product_uos and move_line.product_uos.id or False
				# if not uos_id and inv_type in ('out_invoice', 'out_refund'):
					# uos_id = move_line.product_uom.id

				# account_id = self.pool.get('account.fiscal.position').map_account(cr, uid, partner.property_account_position, account_id)
				# invoice_line_id = invoice_line_obj.create(cr, uid, {
					# 'name': name,
					# 'origin': origin,
					# 'invoice_id': invoice_id,
					# 'uos_id': uos_id,
					# 'product_id': move_line.product_id.id,
					# 'account_id': account_id,
					# 'price_unit': price_unit,
					# 'discount': discount,
					# 'quantity': move_line.product_uos_qty or move_line.product_qty,
					# 'invoice_line_tax_id': [(6, 0, tax_ids)],
					# 'account_analytic_id': account_analytic_id,
				# }, context=context)
				# self._invoice_line_hook(cr, uid, move_line, invoice_line_id)

			# invoice_obj.button_compute(cr, uid, [invoice_id], context=context,
					# set_total=(inv_type in ('in_invoice', 'in_refund')))
			# self.write(cr, uid, [picking.id], {
				# 'invoice_state': 'invoiced',
				# }, context=context)
			# self._invoice_hook(cr, uid, picking, invoice_id)
		# self.write(cr, uid, res.keys(), {
			# 'invoice_state': 'invoiced',
			# }, context=context)
		# return res

stock_picking()