# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2011 Diseño Operativo de Software. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from datetime import datetime
from osv import osv, fields
from tools.translate import _
import netsvc
import time
import tools
import decimal_precision as dp

class mrp_multiproduction(osv.osv):
	"""
	Manufacturing Orders
	"""
	_name = 'mrp.multiproduction'
	_description = 'special Manufacturing Order'
	_date_name  = 'date_planned'
	
	_columns = {
		'name': fields.char('Reference', size=64, required=True),
		'product_id': fields.many2one('product.product', 'Product', states={'done': [('readonly', True)]}, required=True, ),
		'product_qty': fields.float('Product Qty', states={'done': [('readonly', True)]}, required=True, ),
		'product_uom': fields.many2one('product.uom', 'Product UOM', states={'done': [('readonly', True)]}, required=True, ),
		'location_src_id': fields.many2one('stock.location', 'Source Product Location', states={'done': [('readonly', True)]}, required=True,
			help="Location where the system will look for source product."),
		'location_dest_id': fields.many2one('stock.location', 'Finished Products Location', states={'done': [('readonly', True)]}, required=True,
			help="Location where the system will stock the finished products."),
		'date_planned': fields.datetime('Scheduled date', required=True, select=1),
		'date_start': fields.datetime('Start Date', select=True),
		'date_finished': fields.datetime('End Date', select=True),
		'prodlot_id': fields.many2one('stock.production.lot', 'Production Lot', states={'done': [('readonly', True)]}, help="Production lot is used to put a serial number on the production", required=True, select=True), 
		'finished_product_lines': fields.one2many('mrp.multiproduction.product.line', 'order_id', 'Scheduled goods', states={'done': [('readonly', True)]}),
		'consumed_product_lines': fields.one2many('mrp.multiproduction.consumed.product.line', 'order_id', 'Consumed Products', states={'done': [('readonly', True)]}),
		'price_unit': fields.float('Unit Price', digits_compute= dp.get_precision('Account'), help="Technical field used to record the product cost set by the user during a picking confirmation (when average price costing method is used)"),
        'price_currency_id': fields.many2one('res.currency', 'Currency for average price', help="Technical field used to record the currency chosen by the user during a picking confirmation (when average price costing method is used)"),
		'state': fields.selection([('draft','Draft'),('in_production','In Production'),('cancel','Cancelled'),('done','Done')],'State', readonly=True,
									help='When the manufacturing order is created the state is set to \'Draft\'.\n When the production gets started then the state is set to \'In Production\'.\n When the production is over, the state is set to \'Done\'.'),
		'company_id': fields.many2one('res.company','Company',required=True),
	}
	_defaults = {
		'state': lambda *a: 'draft',
		'date_planned': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
		'product_qty':  lambda *a: 1.0,
		'name': lambda x, y, z, c: x.pool.get('ir.sequence').get(y, z, 'mrp.multiproduction') or '/',
		'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'mrp.multiproduction', context=c),
	}
	_order = 'name desc, date_planned asc';
	
	def action_production_end(self, cr, uid, ids, context=None):
		""" Changes production state to Finish and writes finished date.
		@return: True
		"""
		
		if context is None:
			context = {}
		stock_mov_obj = self.pool.get('stock.move')
		
		for multiproduction in self.browse(cr, uid, ids):
			# MOVIMIENTO DE SALIDA
			quantity = multiproduction.product_qty
			move_ids = stock_mov_obj.search(cr, uid, [('location_dest_id', '=', multiproduction.location_src_id.id), ('prodlot_id', '=', multiproduction.prodlot_id.id), ('location_id', '!=', multiproduction.location_src_id.id)])
			for move_src in stock_mov_obj.browse(cr, uid, move_ids, context=context):
				if quantity > 0:
					quantity_to_consume = quantity
					if quantity_to_consume > move_src.product_qty:
						quantity_to_consume = move_src.product_qty
					quantity -= quantity_to_consume
					new_moves_id = stock_mov_obj.action_consume(cr, uid, [move_src.id], quantity_to_consume, multiproduction.location_src_id.id, context=context)
			if quantity > 0:
				locations = self.pool.get('stock.location').search(cr, uid, [('usage', '=', 'customer')], context=context)
				data = {
					'name':'PROD:'+multiproduction.name,
					'date': time.strftime('%Y-%m-%d %H:%M:%S'),
					'product_id': multiproduction.product_id.id,
					'product_qty': quantity,
					'product_uos_qty': quantity,
					'product_uom': multiproduction.product_uom.id,
					'product_uos': multiproduction.product_uom.id,
					'location_id': multiproduction.location_src_id.id,
					'prodlot_id': multiproduction.prodlot_id.id,
					'location_dest_id': locations[0] or multiproduction.location_dest_id.id,
					'state': 'done',
					'company_id': multiproduction.company_id.id,
				}
				newmove_final_id = self.pool.get('stock.move').create(cr, uid, data)
				#print newmove_final_id
			# GENERAR LOS MOVIMIENTOS DE ENTRADA DE LOS PRODUCTOS GENERADOS
			prodlot = self.pool.get('stock.production.lot').browse(cr, uid, multiproduction.prodlot_id.id, context=context)
			#print "print prodlot = "
			#print prodlot
			moves = []
			lot_index = 1
			for line in multiproduction.finished_product_lines:
				prodlot_child_ids = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', line.product_id.id),('prefix', '=', str(prodlot.prefix)),('name', '=', str(prodlot.name))])
				if len(prodlot_child_ids) > 0:
					new_prodlot_id = prodlot_child_ids[0]
					#print "ya existe el lote"
				else:
					prodlot_vals = {
					'name': prodlot.name, 
					'prefix': prodlot.prefix,
					'product_id': line.product_id.id,
					'company_id': multiproduction.company_id.id,
					}
					#print prodlot_vals
					new_prodlot_id = self.pool.get('stock.production.lot').create(cr, uid, prodlot_vals)
				
				source = line.product_id.product_tmpl_id.property_stock_production.id
				data = {
					'name':'PROD:'+multiproduction.name,
					'date': time.strftime('%Y-%m-%d %H:%M:%S'),
					'product_id': line.product_id.id,
					'product_qty': line.product_qty,
					'product_uom': line.product_uom.id,
					'product_uos_qty': line.product_qty,
					'product_uos': line.product_uom.id,
					'location_id': source,
					'prodlot_id': new_prodlot_id,
					'location_dest_id': multiproduction.location_dest_id.id,
					'state': 'done',
					'company_id': multiproduction.company_id.id,
				}
				res_final_id = self.pool.get('stock.move').create(cr, uid, data)
				lot_index = lot_index + 1
			for consprod in multiproduction.consumed_product_lines:
				consumed_move_ids = stock_mov_obj.search(cr, uid, [('location_dest_id', '=', multiproduction.location_src_id.id), ('product_id', '=', consprod.product_id.id), ('location_id', '!=', multiproduction.location_src_id.id)])
				quantity = consprod.product_qty
				for move_src in stock_mov_obj.browse(cr, uid, consumed_move_ids, context=context):
					if quantity > 0:
						quantity_to_consume = quantity
						if quantity_to_consume > move_src.product_qty:
							quantity_to_consume = move_src.product_qty
						quantity -= quantity_to_consume
						new_moves_consumed_ids = stock_mov_obj.action_consume(cr, uid, [move_src.id], quantity_to_consume, multiproduction.location_src_id.id, context=context)
				# quantity_aux = 0
				# for new_move_consumed in stock_mov_obj.browse(cr, uid, new_moves_consumed_ids, context=context):
					# quantity_aux += new_move.product_qty
				#si no se habian existencias suficientes, se hace una salida dejando negativas
				if quantity > 0:
					locations = self.pool.get('stock.location').search(cr, uid, [('usage', '=', 'customer')], context=context)
					data = {
						'name':'PROD:'+multiproduction.name,
						'date': time.strftime('%Y-%m-%d %H:%M:%S'),
						'product_id': consprod.product_id.id,
						'product_qty': quantity,
						'product_uos_qty': quantity,
						'product_uom': consprod.product_uom.id,
						'product_uos': consprod.product_uom.id,
						'location_id': multiproduction.location_src_id.id,
						'location_dest_id': locations[0] or multiproduction.location_dest_id.id,
						'state': 'done',
						'company_id': multiproduction.company_id.id,
					}
					newmove_final_id = self.pool.get('stock.move').create(cr, uid, data)
					# print newmove_final_id
				#print new_moves_consumed
				
		return self.write(cr, uid, ids, {'state': 'done', 'date_finished': time.strftime('%Y-%m-%d %H:%M:%S')})
		
	def action_in_production(self, cr, uid, ids, context=None):
		""" Changes state to In Production and writes starting date.
		@return: True
		"""
		self.write(cr, uid, ids, {'state': 'in_production', 'date_start': time.strftime('%Y-%m-%d %H:%M:%S')})
		
		return True
	
	def action_cancel(self, cr, uid, ids, context=None):
		""" Cancels the manufacturing order and related stock moves.
		@return: True
		"""
		if context is None:
			context = {}
		
		self.write(cr, uid, ids, {'state': 'cancel'})
		return True
		
	def product_id_change(self, cr, uid, ids, product_id, context=None):
		""" Finds UoM of changed product.
		@param product_id: Id of changed product.
		@return: Dictionary of values.
		"""
		if not product_id:
			return {'value': {
				'product_uom': False
			}}
		product = self.pool.get('product.product').browse(cr, uid, product_id, context=context)
		result = {
			'product_uom': product.uom_id and product.uom_id.id or False
		}
		return {'value': result}
	def onchange_lot_id(self, cr, uid, ids, prodlot_id, product_id, context={}):
		if not prodlot_id:
			return {'value': {
				'location_src_id': False,
				'location_dest_id': False,
				'product_qty': 1.0,
				'price_unit': 0.0,
				'price_currency_id': False
				
			}}
		locations = self.pool.get('stock.location').search(cr, uid, [('usage', '=', 'internal')], context=context)
		prodlot = self.pool.get('stock.production.lot').browse(cr, uid, prodlot_id, context=context)
		cr.execute("""select location_dest_id, price_unit, price_currency_id from stock_move where stock_move.prodlot_id = %s 
						and stock_move.state='done' 
						and stock_move.product_id = %s 
						and stock_move.location_dest_id IN %s 
						order by stock_move.date DESC LIMIT 1""", (prodlot_id, product_id, tuple(locations)))
		results = cr.fetchone()
		if results and len(results) > 0:
			#print results
			result = {
				'location_src_id': results[0],
				'location_dest_id': results[0],
				'product_qty': prodlot.stock_available,
				'price_unit': results[1],
				'price_currency_id': results[2]
			}
		else:
			result = {
				'location_src_id': False,
				'location_dest_id': False,
				'product_qty': 1.0,
				'price_unit': 0.0,
				'price_currency_id': False
			}
		
		return {'value': result}		
		
	def finished_product_lines_change(self, cr, uid, order_id, context=None):
		#print order_id
		result = {}
		if order_id:
			result['consumed_product_lines'] = self.pool.get('mrp.multiproduction.consumed.product.line').search(cr, uid, [('order_id', '=', order_id)], context=context)
			#print "se han encontrado lineas"
			#print result['consumed_product_lines']
		return {'value': result }
		
	def location_id_change(self, cr, uid, ids, src, dest, context=None):
		""" Changes destination location if source location is changed.
		@param src: Source location id.
		@param dest: Destination location id.
		@return: Dictionary of values.
		"""
		if dest:
			return {}
		if src:
			return {'value': {'location_dest_id': src}}
		return {}
		
mrp_multiproduction()

class mrp_multiproduction_product_line(osv.osv):
	_name = 'mrp.multiproduction.product.line'
	_description = 'Finished Products from Manufacturing Order'
	
	_columns = {
		'product_id': fields.many2one('product.product', 'Product', required=True),
		'product_qty': fields.float('Product Qty', required=True),
		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
		'order_id': fields.many2one('mrp.multiproduction', 'Manufacturing Order', select=True),
	}
	
	def create(self, cr, user, vals, context=None):
		new_id = super(mrp_multiproduction_product_line, self).create(cr, user, vals, context)
		
		bom_obj = self.pool.get('mrp.bom')
		uom_obj = self.pool.get('product.uom')
		uom = uom_obj.browse(cr, user, vals['product_uom'], context=context)
		bom_id = bom_obj._bom_find(cr, user, vals['product_id'], vals['product_uom'], context)
		cons_line_obj = self.pool.get('mrp.multiproduction.consumed.product.line')
		if bom_id:
			#print "hay lista de materiales "
			bom_point = bom_obj.browse(cr, user, bom_id)
			factor = vals['product_qty'] * uom.factor_inv / bom_point.product_uom.factor
			res = bom_obj._bom_explode(cr, user, bom_point, factor / bom_point.product_qty, context)
			results = res[0]
			for line in results:
				#print line
				data = {
					'order_id': vals['order_id'],
					'product_id': line['product_id'],
					'product_qty': line['product_qty'],
					'bom_source': 1,
					'bom_product_qty': line['product_qty'],
					'product_uom': line['product_uom'],
				}
				cons_line_obj.create(cr, user, data)
		#else:
			#print "no hay lista de materiales"
			
		return new_id
		
	def product_id_change(self, cr, uid, ids, product_id, context=None):
		""" Finds UoM of changed product.
		@param product_id: Id of changed product.
		@return: Dictionary of values.
		"""
		if not product_id:
			return {'value': {
				'product_uom': False
			}}
		product = self.pool.get('product.product').browse(cr, uid, product_id, context=context)
		result = {
			'product_uom': product.uom_id and product.uom_id.id or False
		}
		return {'value': result}
		
mrp_multiproduction_product_line()

class mrp_multiproduction_consumed_product_line(osv.osv):
	_name = 'mrp.multiproduction.consumed.product.line'
	_description = 'Consumed Products from Manufacturing Order'
	
	_columns = {
		'product_id': fields.many2one('product.product', 'Product', required=True),
		'product_qty': fields.float('Product Qty', required=True),
		'bom_product_qty': fields.float('LdM Product Qty'),
		'bom_source': fields.boolean('LdM Source'),
		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
		'order_id': fields.many2one('mrp.multiproduction', 'Manufacturing Order', select=True),
	}
	
	_defaults = {
		'bom_source': lambda *a: 0,
		'bom_product_qty':  lambda *a: 0.0,
	}
	
	def product_id_change(self, cr, uid, ids, product_id, context=None):
		""" Finds UoM of changed product.
		@param product_id: Id of changed product.
		@return: Dictionary of values.
		"""
		if not product_id:
			return {'value': {
				'product_uom': False
			}}
		product = self.pool.get('product.product').browse(cr, uid, product_id, context=context)
		result = {
			'product_uom': product.uom_id and product.uom_id.id or False
		}
		return {'value': result}
		
mrp_multiproduction_consumed_product_line()
