# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Inherit account_invoice to add commission settled and commission settled date"""

from osv import osv, fields
from tools.translate import _
import decimal_precision as dp

class account_invoice(osv.osv):
	"""Inherit account_invoice to add commission settled and commission settled date"""

	_inherit = "account.invoice"
	
	def onchange_partner_id(self, cr, uid, ids, type, partner_id,\
            date_invoice=False, payment_term=False, partner_bank_id=False, company_id=False):
		res = super(account_invoice, self).onchange_partner_id(cr, uid, ids, type, partner_id,\
            date_invoice=date_invoice, payment_term=payment_term, partner_bank_id=partner_bank_id, company_id=company_id)
		if not partner_id:
			res['value']['agent_id'] = False
			res['value']['agent2_id'] = False
			return res
		else:
			addr = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery', 'invoice', 'contact'])
			addr = addr['contact']
			partner = self.pool.get('res.partner.address').browse(cr, uid, addr, context=None) 
			res['value']['agent_id'] = partner.agent_id.id
			res['value']['agent2_id'] = False
		
		return res
	
	def onchange_address(self, cr, uid, ids, addr):
		"""extend this event for add agents partner"""
		#res = self.onchange_partner_id(cr, uid, ids, part)
		res = {}
		if not addr:	
			res['agent_id'] = False
			res['agent2_id'] = False
			return res
		else:		
			partner = self.pool.get('res.partner.address').browse(cr, uid, addr, context=None)
			address = self.pool.get('res.partner').address_get(cr, uid, [partner.partner_id.id], ['delivery', 'invoice', 'contact'])
			res['agent_id'] = partner.agent_id.id
			res['agent2_id'] = False
		return {'value': res}
	
	def _get_agent_percentage_distribution_commission1(self, cr, uid, ids, prop, unknow_none, context=None):
		result = {}
		for inv in self.browse(cr, uid, ids, context=context):
			if inv.agent2_percentage_distribution_commission:
				result[inv.id] = 100.0 - inv.agent2_percentage_distribution_commission
			else:
				result[inv.id] = 100.0
		return result
	
	def _invoice_paid_date(self, cr, uid, ids, prop, unknow_none, context=None):
		""" Finds invoice paid date.
		@param prop: Name of field.
		@param unknow_none:
		@return: Dictionary of values.
		"""
		result = {}
		for inv in self.browse(cr, uid, ids, context=context):
			if inv.state == 'paid' and len(inv.payment_ids):
				#result[inv.id] = inv.date_due
				for payment in inv.payment_ids:
					if len(result)>0 and inv.id in result:
						if payment.date > result[inv.id]:
							result[inv.id] = payment.date
					else:
						result[inv.id] = payment.date
			else:
				result[inv.id] = False
		return result
		
	_columns = {
		'decorator_discount': fields.float('Dto decorador'),
		'commission_total': fields.float('Total Commission'),
		'commission_settled_date': fields.datetime('Commission settled date'),
		'commission_settled': fields.boolean('Settled'),
		'agent_id': fields.many2one('sale.agent','Sales Agent'),
		'paid_date': fields.function(_invoice_paid_date, method=True,  type='date', string='Paid Date'),
		'agent2_percentage_distribution_commission': fields.float('% Dist. Com. 2'),
		'agent1_percentage_distribution_commission': fields.function(_get_agent_percentage_distribution_commission1, method=True, type='float', string='% Dist. Com. 1'),
		'agent2_id': fields.many2one('sale.agent','Sales Agent 2'),
	}
	
	def onchange_agent_percentage_distribution_commission2(self, cr, uid, ids, agent2_percentage_distribution_commission):
		v = {}
		perc = 100
		if agent2_percentage_distribution_commission:
			perc = 100.0 - agent2_percentage_distribution_commission
			if perc < 0:
				perc = 0
			if perc > 100:
				perc = 100
		v['agent1_percentage_distribution_commission'] = perc
		return {'value': v}
		
account_invoice()