# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from datetime import datetime
from report import report_sxw
import pooler
import decimal_precision as dp
from dateutil.relativedelta import relativedelta

class report_sale_agent(report_sxw.rml_parse):
	def __init__(self, cr, uid, name, context):
		super(report_sale_agent, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({
			'time': time,
			'get_agent': self.get_agent,
			'get_date_start': self.get_date_start,
			'get_date_end': self.get_date_end,
			'get_agent_lines': self.get_agent_lines,
			'get_totals_agent_lines': self.get_totals_agent_lines,
		})
		self.context = context
		self.data_report = context['data_report']
		user = self.pool.get('res.users').browse(cr, uid, uid)
		self.company = user.company_id
		
	def get_agent(self, form):
		agent = form['agent']
		return agent
		
	def get_date_start(self, form):
		date_start = form['date_start']
		return date_start
	
	def get_date_end(self, form):
		date_end = form['date_end']
		return date_end
	
	def get_agent_lines(self, form):
		lines = []
		
		date_start = form['date_start']
		date_end = form['date_end']
		agent = form['agent'] or ''
		company_id = str(self.company.id)
		
		sql = "SELECT id, name, sum(amount_untaxed) As amount_untaxed, sum(amount_total) As amount_total "
		sql += "FROM  " 
		sql += "(  "
		sql += "	SELECT a.id, a.name,  "
		sql += "	CASE  "
		sql += "		WHEN i.agent2_id IS NULL OR COALESCE(agent2_percentage_distribution_commission,0) = 0 "
		sql += "			THEN sum(CASE WHEN i.type IN ('out_refund') THEN i.amount_untaxed * -1 ELSE i.amount_untaxed END) "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_untaxed * -1 ELSE i.amount_untaxed END) * (1 - (agent2_percentage_distribution_commission / 100))) "
		sql += "	END As amount_untaxed,  "
		sql += "	CASE "
		sql += "		WHEN i.agent2_id IS NULL OR COALESCE(agent2_percentage_distribution_commission,0) = 0 "
		sql += "			THEN sum(CASE WHEN i.type IN ('out_refund') THEN i.amount_total * -1 ELSE i.amount_total END) "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_total * -1 ELSE i.amount_total END) * (1 - (agent2_percentage_distribution_commission / 100))) "
		sql += "	END As amount_total  "
		sql += "	FROM sale_agent a  "
		sql += "	INNER JOIN account_invoice i ON a.id=i.agent_id  "
		sql += "	WHERE UPPER(a.name) LIKE UPPER('%" + agent + "%')   "
		sql += "	AND i.date_invoice >= '" + date_start + "' AND i.date_invoice <= '" + date_end + "'  "
		sql += "	AND a.company_id=" + company_id + " "
		sql += "	AND i.type IN ('out_invoice', 'out_refund') "
		sql += "	GROUP BY a.id, a.name, i.agent2_id, i.agent2_percentage_distribution_commission "
		sql += "	UNION  "
		sql += "	SELECT a.id, a.name, "
		sql += "	CASE  "
		sql += "		WHEN COALESCE(agent2_percentage_distribution_commission,0) = 0 THEN 0 "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_untaxed * -1 ELSE i.amount_untaxed END) * (agent2_percentage_distribution_commission / 100)) "
		sql += "	END As amount_untaxed,  "
		sql += "	CASE	 "
		sql += "		WHEN COALESCE(agent2_percentage_distribution_commission,0) = 0 THEN 0 "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_total * -1 ELSE i.amount_total END) * (agent2_percentage_distribution_commission / 100)) "
		sql += "	END As amount_total "
		sql += "	FROM sale_agent a  "
		sql += "	INNER JOIN account_invoice i ON a.id=i.agent2_id  "
		sql += "	WHERE UPPER(a.name) LIKE UPPER('%" + agent + "%')   "
		sql += "	AND i.date_invoice >= '" + date_start + "' AND i.date_invoice <= '" + date_end + "'  "
		sql += "	AND a.company_id=" + company_id + " "
		sql += "	AND i.type IN ('out_invoice', 'out_refund') "
		sql += "	GROUP BY a.id, a.name, i.agent2_id, i.agent2_percentage_distribution_commission "
		sql += ") AS T  "
		sql += "GROUP BY id, name  "
		sql += "ORDER BY name  "

		self.cr.execute(sql)
		
		result = self.cr.dictfetchall()
		
		for res in result:
			
			line = {}
				
			line["name"] = res["name"]
			line["amount_untaxed"] = res["amount_untaxed"]
			line["amount_total"] = res["amount_total"]

			lines.append(line)

		return lines
	
	
	def get_totals_agent_lines(self, form):
		
		date_start = form['date_start']
		date_end = form['date_end']
		agent = form['agent'] or ''
		company_id = str(self.company.id)
		
		sql = "SELECT sum(amount_untaxed) As amount_untaxed, sum(amount_total) As amount_total "
		sql += "FROM  " 
		sql += "(  "
		sql += "	SELECT a.id, a.name,  "
		sql += "	CASE  "
		sql += "		WHEN i.agent2_id IS NULL OR COALESCE(agent2_percentage_distribution_commission,0) = 0 "
		sql += "			THEN sum(CASE WHEN i.type IN ('out_refund') THEN i.amount_untaxed * -1 ELSE i.amount_untaxed END) "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_untaxed * -1 ELSE i.amount_untaxed END) * (1 - (agent2_percentage_distribution_commission / 100))) "
		sql += "	END As amount_untaxed,  "
		sql += "	CASE "
		sql += "		WHEN i.agent2_id IS NULL OR COALESCE(agent2_percentage_distribution_commission,0) = 0 "
		sql += "			THEN sum(CASE WHEN i.type IN ('out_refund') THEN i.amount_total * -1 ELSE i.amount_total END) "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_total * -1 ELSE i.amount_total END) * (1 - (agent2_percentage_distribution_commission / 100))) "
		sql += "	END As amount_total  "
		sql += "	FROM sale_agent a  "
		sql += "	INNER JOIN account_invoice i ON a.id=i.agent_id  "
		sql += "	WHERE UPPER(a.name) LIKE UPPER('%" + agent + "%')   "
		sql += "	AND i.date_invoice >= '" + date_start + "' AND i.date_invoice <= '" + date_end + "'  "
		sql += "	AND a.company_id=" + company_id + " "
		sql += "	AND i.type IN ('out_invoice', 'out_refund') "
		sql += "	GROUP BY a.id, a.name, i.agent2_id, i.agent2_percentage_distribution_commission "
		sql += "	UNION  "
		sql += "	SELECT a.id, a.name, "
		sql += "	CASE  "
		sql += "		WHEN COALESCE(agent2_percentage_distribution_commission,0) = 0 THEN 0 "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_untaxed * -1 ELSE i.amount_untaxed END) * (agent2_percentage_distribution_commission / 100)) "
		sql += "	END As amount_untaxed,  "
		sql += "	CASE	 "
		sql += "		WHEN COALESCE(agent2_percentage_distribution_commission,0) = 0 THEN 0 "
		sql += "		ELSE sum((CASE WHEN i.type IN ('out_refund') THEN i.amount_total * -1 ELSE i.amount_total END) * (agent2_percentage_distribution_commission / 100)) "
		sql += "	END As amount_total "
		sql += "	FROM sale_agent a  "
		sql += "	INNER JOIN account_invoice i ON a.id=i.agent2_id  "
		sql += "	WHERE UPPER(a.name) LIKE UPPER('%" + agent + "%')   "
		sql += "	AND i.date_invoice >= '" + date_start + "' AND i.date_invoice <= '" + date_end + "'  "
		sql += "	AND a.company_id=" + company_id + " "
		sql += "	AND i.type IN ('out_invoice', 'out_refund') "
		sql += "	GROUP BY a.id, a.name, i.agent2_id, i.agent2_percentage_distribution_commission "
		sql += ") AS T  "
		
		self.cr.execute(sql)
		
		result = self.cr.dictfetchall()
		
		line = {}
		
		for res in result:
			line["amount_untaxed"] = res["amount_untaxed"]
			line["amount_total"] = res["amount_total"]
			
		return line
	
		
report_sxw.report_sxw('report.sale_agent_report', 'sale.agent', 'addons/dos_sale_commissioncalc/report/report_sale_agent.rml', parser=report_sale_agent)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

