# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from datetime import datetime
from report import report_sxw
import pooler
import decimal_precision as dp
from dateutil.relativedelta import relativedelta

class report_sale_commissions(report_sxw.rml_parse):
	def __init__(self, cr, uid, name, context):
		super(report_sale_commissions, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({
			'time': time,
			'get_date_start': self.get_date_start,
			'get_date_end': self.get_date_end,
			'get_agent_invoices': self.get_agent_invoices,
			'get_total_agent_amount_untaxed': self.get_total_agent_amount_untaxed,
			'get_agent_receipts_pendings': self.get_agent_receipts_pendings,
			'get_total_agent_receipts_pendings': self.get_total_agent_receipts_pendings,
			'get_total_agent_sales_in_period': self.get_total_agent_sales_in_period,
			'get_commission_average': self.get_commission_average,
			'get_commission_total': self.get_commission_total,
			'get_invoice_commission_total': self.get_invoice_commission_total,
			'get_invoice_commission_percentage': self.get_invoice_commission_percentage,
			'get_total_agent_receipts_pendings_before_period': self.get_total_agent_receipts_pendings_before_period,
			'get_inv_amount_untaxed': self.get_inv_amount_untaxed,
			'get_line_discount': self.get_line_discount,
		})
		self.context = context
		self.data_report = context['data_report']
		
	def get_date_start(self, form):
		date_start = form['date_start']
		return date_start
	
	def get_date_end(self, form):
		date_end = form['date_end']
		return date_end
	
	def get_agent_invoices(self, form, agent):
		pool = pooler.get_pool(self.cr.dbname)
		inv_obj = pool.get('account.invoice')
		inv_ids = self.data_report['agent_invoices'][agent.id]
		
		# if form['settled'] == 'all':
			# inv_ids = inv_obj.search(self.cr, self.uid, [ ('state','=', 'paid'), ('date_invoice','>=',form['date_start']), ('date_invoice','<=',form['date_end']), ('paid_date','>=',form['payment_date_start']), ('paid_date','<=',form['payment_date_end']), ('agent_id', '=', agent.id) ])
		# else:
			# settled = False
			# if form['settled'] == 'settled':
				# settled = True
			# inv_ids = inv_obj.search(self.cr, self.uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','>=',form['date_start']), ('date_invoice','<=',form['date_end']), ('paid_date','>=',form['payment_date_start']), ('paid_date','<=',form['payment_date_end']), ('agent_id', '=', agent.id)])
		invoice_ids = inv_obj.browse(self.cr, self.uid, inv_ids)
		return invoice_ids
		
	def get_total_agent_amount_untaxed(self, form, agent):
		invoice_ids = self.get_agent_invoices(form, agent)
		total = 0.0
		for inv in invoice_ids:
			total += inv.amount_untaxed
			#si se ha a�adido el articulo de entrega a cuenta, hay que sumar lo a la base imponible
			for line in inv.invoice_line:
				if line.product_id and line.product_id.name == 'ENTREGA A CUENTA':
					total -= line.price_subtotal #resto porque es negativo, con lo que debo de estar sumando
					
		return total
	def get_inv_amount_untaxed(self, invoice):
		total = invoice.amount_untaxed
		for line in invoice.invoice_line:
			if line.product_id and line.product_id.name == 'ENTREGA A CUENTA':
				total -= line.price_subtotal
		if invoice.type == 'out_refund':
			total = -total
		return total
	
	def get_agent_receipts_pendings(self, form, agent):
		pool = pooler.get_pool(self.cr.dbname)
		inv_obj = pool.get('account.invoice')
		#inv_ids = inv_obj.search(self.cr, self.uid, [ ('state','=', 'open'), ('commission_settled','=', False), ('agent_id', '=', agent.id)])
		inv_ids = self.data_report['agent_invoice_pending'][agent.id]
		invoice_ids = inv_obj.browse(self.cr, self.uid, inv_ids)
		receipts = []
		for inv in invoice_ids:
			data_lines = [x for x in inv.move_id.line_id if x.account_id.id == inv.account_id.id]
			for move_line in data_lines:
				if not move_line.reconcile_id:
					if inv.type == 'out_refund':
						move_line.amount_residual = -move_line.amount_residual
					receipts.append(move_line)
		
		return receipts
	
	def get_total_agent_receipts_pendings(self, form, agent):
		move_ids = self.get_agent_receipts_pendings(form, agent)
		total = 0.0
		for move in move_ids:
			total += move.amount_residual
		return total
	
	def get_total_agent_receipts_pendings_before_period(self, form, agent):
		total = 0.0
		pool = pooler.get_pool(self.cr.dbname)
		inv_obj = pool.get('account.invoice')
		inv_ids = inv_obj.search(self.cr, self.uid, [ ('state','=', 'open'), ('commission_settled','=', False), ('agent_id', '=', agent.id)])
		invoice_ids = inv_obj.browse(self.cr, self.uid, inv_ids)
		receipts = []
		for inv in invoice_ids:
			if inv.date_invoice < form['date_start']:
				data_lines = [x for x in inv.move_id.line_id if x.account_id.id == inv.account_id.id]
				for move_line in data_lines:
					if not move_line.reconcile_id:
						total += move_line.amount_residual
		
		return total
	def get_total_agent_sales_in_period(self, form, agent):
		pool = pooler.get_pool(self.cr.dbname)
		inv_obj = pool.get('account.invoice')
		
		if form['settled'] == 'all':
			inv_ids = inv_obj.search(self.cr, self.uid, [ ('state','=', 'paid'), ('date_invoice','>=',form['date_start']), ('date_invoice','<=',form['date_end']), ('paid_date','>=',form['payment_date_start']), ('paid_date','<=',form['payment_date_end']), ('agent_id', '=', agent.id) ])
		else:
			settled = False
			if form['settled'] == 'settled':
				settled = True
				
		inv_ids = inv_obj.search(self.cr, self.uid, [ ('state','in', ['open', 'paid']), ('date_invoice','>=',form['date_start']), ('date_invoice','<=',form['date_end']), ('agent_id', '=', agent.id)])
		invoice_ids = inv_obj.browse(self.cr, self.uid, inv_ids)
		
		total = 0.0
		for inv in invoice_ids:
			total += inv.amount_untaxed
		return total
	
	def get_invoice_commission_percentage(self, invoice, agent_obj):
		if not invoice.agent_id:
			return 0.0
		dto = invoice.header_discount + invoice.early_payment_discount + self.get_line_discount(invoice)
		
		if invoice.decorator_discount:
			dto = invoice.decorator_discount
			
		final_date = invoice.date_invoice
		if invoice.paid_date and invoice.paid_date > final_date:
			final_date = invoice.paid_date
#		if invoice.date_due and invoice.date_due > final_date:
#			final_date = invoice.date_due
		diasMes = 30
		
		dtt1 = datetime.strptime(final_date, '%Y-%m-%d')
		dtt2 = datetime.strptime(invoice.date_invoice, '%Y-%m-%d')
		dif = dtt1 - dtt2
		
		numPeriodos30dias = round(dif.days / 30,0)
		agent = invoice.agent_id
		por_commission = 0.0
		if agent.commissionwithouttable:
			if dto == 10.0:
				por_commission = 4.0
			elif dto == 15.0:
				por_commission = 2.0
			elif dto == 20.0:
				por_commission = 1.0
			else:
				por_commission = agent.commission_rate
		else:
			if invoice.partner_id.unique_commission:
				por_commission = invoice.partner_id.unique_commission
			elif invoice.partner_id.unique_commission_2:
				por_commission = invoice.partner_id.unique_commission_2
			elif agent.commission_rate:
				por_commission = agent.commission_rate
			elif agent.commissiontozeroforce:
				por_commission = 0.0
			else:
				if agent.commissiontable_id:
					if dto == 0.0:
						por_commission = agent.commissiontable_id.commissiondctzero
					else:
						dto_assign = 0
						line_assign = False
						for commission_line in agent.commissiontable_id.sale_commissionlines:
							if commission_line.discount >= dto and (not line_assign or dto_assign > commission_line.discount):
								dto_assign = commission_line.discount
								por_commission = commission_line.commission
								line_assign = True
						if not line_assign:
							if dto <= agent.commissiontable_id.maxdctpercent:
								por_commission = agent.commissiontable_id.maxdctcommission
							else:
								por_commission = 0.0
					
					por_commission = por_commission - (numPeriodos30dias * agent.commissiontable_id.dctbypaymentdelay)
					if por_commission < agent.commissiontable_id.maxdctcommission:
						por_commission = agent.commissiontable_id.maxdctcommission
		
		if invoice.agent2_id and invoice.agent2_percentage_distribution_commission > 0:
			por_commission2 = por_commission * invoice.agent2_percentage_distribution_commission / 100.0
			por_commission2 = round(por_commission2,2)
			por_commission = por_commission - por_commission2
			if invoice.agent2_id.id == agent_obj.id:
				por_commission = por_commission2
		
		return por_commission
	
	def get_invoice_commission_total(self, invoice, agent):
		por_commission = self.get_invoice_commission_percentage(invoice, agent)
		base_amount = invoice.amount_untaxed
		#si se ha a�adido el articulo de entrega a cuenta, hay que sumar lo a la base imponible
		for line in invoice.invoice_line:
			if line.product_id and line.product_id.name == 'ENTREGA A CUENTA':
				base_amount -= line.price_subtotal #resto porque es negativo, con lo que debo de estar sumando
		# if invoice.decorator_discount:
			# base_amount = invoice.amount_untaxed * (1.0 - (decorator_discount / 100.0))
			# base_amount = round(base_amount,2)
		total = base_amount * por_commission / 100.0
		total = round(total,2)
		if invoice.type == 'out_refund':
			total = -total
		return total
	
	def get_commission_total(self, form, agent):
		invoice_ids = self.get_agent_invoices(form, agent)
		total = 0.0
		for inv in invoice_ids:
			total += self.get_invoice_commission_total(inv, agent)
		
		return total
	
	def get_commission_average(self, form, agent):
		base = self.get_total_agent_amount_untaxed(form, agent)
		commission = self.get_commission_total(form, agent)
		
		if base == 0.0:
			return 0.0
		else:
			return (commission / base) * 100.0
		
	def get_line_discount(self, invoice):
		
		total_bruto = 0.0
		total = 0.0
		discount = 0
		 
		for line in invoice.invoice_line:
			if line.discount:
				total_bruto += line.price_unit * line.quantity
				total += line.price_subtotal
		
		if total != 0.0 and total_bruto != 0.0:
			discount = (1-(total / total_bruto ))*100
		return discount
		
report_sxw.report_sxw('report.sale_commissions_report', 'sale.agent', 'addons/dos_sale_commissioncalc/report/report_sale_commissions.rml', parser=report_sale_commissions)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

