# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Modificamos el pedido para que al generar la factura herede el representante del cliente"""

from osv import fields, osv
from tools.translate import _

class sale_order(osv.osv):
	"""Modificamos las ventas para incluir el comportamiento de comisiones"""

	_inherit = "sale.order"
	
	def _get_percentage_commission_agent(self, cr, uid, ids, prop, unknow_none, context=None):
		result = {}
		for order in self.browse(cr, uid, ids, context=context):
			if order.agent2_percentage_distribution_commission:
				result[order.id] = 100.0 - order.agent2_percentage_distribution_commission
			else:
				result[order.id] = 100.0
		return result
		
	_columns = {
		'agent2_percentage_distribution_commission': fields.float('% Dist. Com. 2'),
		'agent1_percentage_distribution_commission': fields.function(_get_percentage_commission_agent, method=True, type='float', string='% Dist. Com. 1'),
		'agent_id': fields.many2one('sale.agent','Sales Agent'),
		'agent2_id': fields.many2one('sale.agent','Sales Agent 2'),
	}
	
	def onchange_partner_id(self, cr, uid, ids, part):
		"""extend this event for add agents partner"""
		#res = self.onchange_partner_id(cr, uid, ids, part)
		res = super(sale_order, self).onchange_partner_id(cr, uid, ids, part)
		if not part:
			res['value']['agent_id'] = False
			res['value']['agent2_id'] = False
			return res
		else:
			addr = self.pool.get('res.partner').address_get(cr, uid, [part], ['delivery', 'invoice', 'contact'])
			addr = addr['delivery']
			partner = self.pool.get('res.partner.address').browse(cr, uid, addr, context=None) 
			res['value']['agent_id'] = partner.agent_id.id
			res['value']['agent2_id'] = False
		
		return res
		
	def onchange_shipping_address(self, cr, uid, ids, addr):
		"""extend this event for add agents partner"""
		#res = self.onchange_partner_id(cr, uid, ids, part)
		res = {}
		if not addr:	
			res['agent_id'] = False
			res['agent2_id'] = False
			return res
		else:		
			partner = self.pool.get('res.partner.address').browse(cr, uid, addr, context=None)
			address = self.pool.get('res.partner').address_get(cr, uid, [partner.partner_id.id], ['delivery', 'invoice', 'contact'])
			res['partner_shipping_id'] = address['delivery']
			res['agent_id'] = partner.agent_id.id
			res['agent2_id'] = False
		return {'value': res}
		
	# def onchange_partner_id_commission(self, cr, uid, ids, part):
		# """extend this event for add agents partner"""
		# res = self.onchange_partner_id(cr, uid, ids, part)
		# if not part:
			# res['value']['agent_id'] = False
			# res['value']['agent2_id'] = False
			# return res
		# else:
			# partner = self.pool.get('res.partner').browse(cr, uid, part, context=None)
			# res['value']['agent_id'] = partner.agent_id.id
			# res['value']['agent2_id'] = False
			
		# return res
		
	def onchange_agent_percentage_distribution_commission2(self, cr, uid, ids, agent2_percentage_distribution_commission):
		v = {}
		perc = 100
		if agent2_percentage_distribution_commission:
			perc = 100.0 - agent2_percentage_distribution_commission
			if perc < 0:
				perc = 0
			if perc > 100:
				perc = 100
		v['agent1_percentage_distribution_commission'] = perc
		return {'value': v}
		
	def action_invoice_create(self, cr, uid, ids, grouped=False, states=['confirmed', 'done', 'exception'], date_inv = False, context=None):
		"""
		Inherited method for writing agent_id partner value in created invoice
		"""
		invoice_id = super(sale_order, self).action_invoice_create(cr, uid, ids, grouped=grouped, states=states, date_inv = date_inv, context=context)
		inv_obj = self.pool.get('account.invoice')
		
		for saleorder in self.browse(cr, uid, ids, context=context):
			if saleorder.partner_id:
				for invoice in saleorder.invoice_ids:
					inv_obj.write(cr, uid, [invoice.id], {
						'agent_id': saleorder.agent_id and saleorder.agent_id.id or False, 
						'agent2_id': saleorder.agent2_id and saleorder.agent2_id.id or False , 
						'agent2_percentage_distribution_commission': saleorder.agent2_percentage_distribution_commission
						})
				
		return invoice_id
	

sale_order()