# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from osv import osv, fields
from datetime import datetime
from dateutil.relativedelta import relativedelta
from tools.translate import _

class dos_sale_commission_report_wizard(osv.osv_memory):
	_name = 'dos.sale.commission.report.wizard'
	_description = 'Sale Commissions Report'
	
	def _get_first_day_present_year(self, cr, uid, context=None):
		if context is None:
			context = {}
		date_ref = datetime.now().strftime('%Y-%m-%d')
		first_date = (datetime.strptime(date_ref, '%Y-%m-%d') + relativedelta(day=1, month=1))
		return first_date.strftime('%Y-%m-%d')
		# return datetime.now()
	
	_columns = {
		'date_start': fields.date('From invoice Date' ),
		'date_end': fields.date('To invoice Date' ),
		'payment_date_start': fields.date('From payment date'),
		'payment_date_end': fields.date('To payment date'),
		'settled': fields.selection([
			('all','All'),
			('not_settled','Not Settled'),
			('settled','Settled'),
			],'Settled' ),
		'final_report': fields.boolean('Definitivo'),
		'agent_id': fields.many2one('sale.agent','Sales Agent'),
	}
	
	_defaults = {
		'date_start': _get_first_day_present_year,
		'date_end': time.strftime('%Y-%m-%d'),
		'payment_date_start': _get_first_day_present_year,
		'payment_date_end': time.strftime('%Y-%m-%d'),
		'settled': 'not_settled',
		'final_report': False,
	}
	
	def get_invoice_commission_percentage(self, invoice, agent_obj):
		if not invoice.agent_id:
			return 0.0
			
		dto = invoice.header_discount + invoice.early_payment_discount
		if invoice.decorator_discount:
			dto = invoice.decorator_discount
		final_date = invoice.date_invoice
		if invoice.paid_date and invoice.paid_date > final_date:
			final_date = invoice.paid_date
		if invoice.date_due and invoice.date_due > final_date:
			final_date = invoice.date_due
		diasMes = 30
		
		dtt1 = datetime.strptime(final_date, '%Y-%m-%d')
		dtt2 = datetime.strptime(invoice.date_invoice, '%Y-%m-%d')
		dif = dtt1 - dtt2
		
		numPeriodos30dias = round(dif.days / 30,0)
		
		agent = invoice.agent_id
		
		por_commission = 0.0
		if agent.commissionwithouttable:
			if dto == 10.0:
				por_commission = 4.0
			elif dto == 15.0:
				por_commission = 2.0
			elif dto == 20.0:
				por_commission = 1.0
			else:
				por_commission = agent.commission_rate
		else:
			if invoice.partner_id.unique_commission:
				por_commission = invoice.partner_id.unique_commission
			elif invoice.partner_id.unique_commission_2:
				por_commission = invoice.partner_id.unique_commission_2
			elif agent.commission_rate:
				por_commission = agent.commission_rate
			elif agent.commissiontozeroforce:
				por_commission = 0.0
			else:
				if agent.commissiontable_id:
					if dto == 0.0:
						por_commission = agent.commissiontable_id.commissiondctzero
					else:
						dto_assign = 0
						line_assign = False
						for commission_line in agent.commissiontable_id.sale_commissionlines:
							if commission_line.discount >= dto and (not line_assign or dto_assign > commission_line.discount):
								dto_assign = commission_line.discount
								por_commission = commission_line.commission
								line_assign = True
						if not line_assign:
							if dto <= agent.commissiontable_id.maxdctpercent:
								por_commission = agent.commissiontable_id.maxdctcommission
							else:
								por_commission = 0.0

					por_commission = por_commission - (numPeriodos30dias * agent.commissiontable_id.dctbypaymentdelay)
					if por_commission < agent.commissiontable_id.maxdctcommission:
						por_commission = agent.commissiontable_id.maxdctcommission
		
		#En la factura pondremos la comision total
		# if invoice.agent2_id and invoice.agent2_percentage_distribution_commission > 0:
			# por_commission2 = por_commission * invoice.agent2_percentage_distribution_commission / 100.0
			# por_commission2 = round(por_commission2,2)
			# por_commission = por_commission - por_commission2
			# if invoice.agent2_id.id == agent_obj.id:
				# por_commission = por_commission2
		
		return por_commission
		
	def _get_agent_ids(self, cr, uid, data, context=None):
		
		inv_obj = self.pool.get('account.invoice')
		cur_obj = self.pool.get('res.currency')
		inv_ids = []
		
				
		# JOSE 2012-03-28 
		#  En Impacto quieren que cuando se saque el informe de no liquidadas, para la base imponible a comisionar y su calculo, 
		#     incluir todas las facturas no liquidadas y filtrar solo hasta la fecha factura indicada, descartando el filtro desde fecha factura
		
		one_agent_to_settled = False
		agent_ids = {}
		if data['agent_id'] and data['settled'] != 'settled' and data['settled'] != 'all':
			agent_ids = self.get_agents_to_settled(cr, uid, data, context)
			one_agent_to_settled = True
			
		if data['agent_id'] and not one_agent_to_settled:
			if data['settled'] == 'all':
				inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end']), ('agent_id', '=', data['agent_id']) ])
			else:
				settled = False
				if data['settled'] == 'settled':
					settled = True
					inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end']), ('agent_id', '=', data['agent_id'])])
				else:
					inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','<=',data['date_end']), ('agent_id', '=', data['agent_id']), ('paid_date','<=',data['payment_date_end'])])
		else:
			if data['settled'] == 'all':
				inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end'])])
			else:
				settled = False
				if data['settled'] == 'settled':
					settled = True
					inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end'])])
				else:
					inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','<=',data['date_end']),('paid_date','<=',data['payment_date_end']) ]) 
				
		inv_ids2 = []
		#para las facturas donde este como segundo representante
		if data['agent_id'] and not one_agent_to_settled:
			if data['settled'] == 'all':
				inv_ids2 = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end']), ('agent2_id', '=', data['agent_id']) ])
			else:
				settled = False
				if data['settled'] == 'settled':
					settled = True
					inv_ids2 = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end']), ('agent2_id', '=', data['agent_id'])])
				else:
					inv_ids2 = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','<=',data['date_end']), ('agent2_id', '=', data['agent_id']),('paid_date','<=',data['payment_date_end'])])
					
			inv_ids = inv_ids + inv_ids2
				
		res = {}
		invoice_ids = inv_obj.browse(cr,uid,inv_ids)
		
		agent_invoices = {}
		for invoice in invoice_ids:
			
			if invoice.paid_date<=data['payment_date_end']:
			
				add_invoice = True
				
				if invoice.invoice_line and len(invoice.invoice_line) == 1:
					for line in invoice.invoice_line:
						if line.product_id and line.product_id.name == 'ENTREGA A CUENTA':
							add_invoice = False
				if one_agent_to_settled:
					if invoice.agent_id.id not in agent_ids:
						add_invoice = False
						
				if add_invoice:
					if data['agent_id'] and not one_agent_to_settled:
						res[data['agent_id']] = data['agent_id']
						if not data['agent_id'] in agent_invoices:
							agent_invoices[data['agent_id']] = [invoice.id]
						else:
							agent_invoices[data['agent_id']].append(invoice.id)
					else:
						
						if invoice.agent_id and not invoice.agent_id.id in res:
							res[invoice.agent_id.id] = invoice.agent_id.id
						if invoice.agent2_id and not invoice.agent2_id.id in res:
							res[invoice.agent2_id.id] = invoice.agent2_id.id
						
						if invoice.agent_id:
							if not invoice.agent_id.id in agent_invoices:
								agent_invoices[invoice.agent_id.id] = [invoice.id]
							else:
								agent_invoices[invoice.agent_id.id].append(invoice.id)
						
						if invoice.agent2_id:
							if not invoice.agent2_id.id in agent_invoices:
								agent_invoices[invoice.agent2_id.id] = [invoice.id]
							else:
								agent_invoices[invoice.agent2_id.id].append(invoice.id)
						
						if data['final_report'] and not invoice.commission_settled:
							commission = self.get_invoice_commission_percentage(invoice, invoice.agent_id)
							invoice.commission_settled = True
							invoice.commission_settled_date = time.strftime('%Y-%m-%d')
							base_amount = invoice.amount_untaxed
							#si se ha a?adido el articulo de entrega a cuenta, hay que sumar lo a la base imponible
							for line in invoice.invoice_line:
								if line.product_id and line.product_id.name == 'ENTREGA A CUENTA':
									base_amount -= line.price_subtotal #resto porque es negativo, con lo que debo de estar sumando
								
							invoice.commission_total = cur_obj.round(cr, uid, invoice.currency_id, commission * base_amount / 100.0)
							inv_obj.write(cr, uid, [invoice.id], {'commission_settled':invoice.commission_settled, 'commission_settled_date': invoice.commission_settled_date, 'commission_total':invoice.commission_total})
					
		data['agent_invoices'] = agent_invoices
		
		agent_invoice_pending = {}
		for agent_id in res.values():
			inv_pending_ids = inv_obj.search(cr, uid, [ ('state','=', 'open'), ('commission_settled','=', False), ('date_invoice','<=',data['date_end']), ('agent_id', '=', agent_id)])
			inv_pending2_ids = inv_obj.search(cr, uid, [ ('state','=', 'open'), ('commission_settled','=', False), ('date_invoice','<=',data['date_end']), ('agent2_id', '=', agent_id)])
			inv_pending_ids = inv_pending_ids + inv_pending2_ids
			agent_invoice_pending[agent_id] = inv_pending_ids
		
		data['agent_invoice_pending'] = agent_invoice_pending
		
		return res.values()
	
	def get_agents_to_settled(self, cr, uid, data, context = None):
		inv_obj = self.pool.get('account.invoice')
		inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', False), ('date_invoice','<=',data['date_end']) ])
		invoice_ids = inv_obj.browse(cr,uid,inv_ids)
		agent_ids = {}
		agent_ids[data["agent_id"]] = data["agent_id"]
		for inv in invoice_ids:
			if inv.agent_id and inv.agent_id.id == data["agent_id"]:
				if inv.agent2_id and inv.agent2_id != inv.agent_id and inv.agent2_id not in agent_ids:
					agent_ids[inv.agent2_id] = inv.agent2_id
			if inv.agent2_id and inv.agent2_id.id == data["agent_id"]:
				if inv.agent_id and inv.agent_id != inv.agent2_id and inv.agent_id not in agent_ids:
					agent_ids[inv.agent_id] = inv.agent_id
		return agent_ids
		
		
	def check_report(self, cr, uid, ids, context=None):
		datas = {}
		if context is None:
			context = {}
		
		data = self.read(cr, uid, ids, [], context=context)[0]
		agent_ids = self._get_agent_ids(cr, uid, data, context=context)
		
		context['data_report'] = data
		datas = {
			 'ids': agent_ids, #context.get('active_ids',[]),
			 'model': 'sale.agent',
			 'form': data,
			 'context': context
		}
		if agent_ids and len(agent_ids) > 0:
			return {
				'type': 'ir.actions.report.xml',
				'report_name': 'sale_commissions_report',
				'context': context,
				'datas': datas,
			}
		else:
			raise osv.except_osv(_('Warning !'),'No hay facturas en los limites indicados.')
			#msgalert = {'title':'Warning','message':'No hay facturas en los limites indicados'}
			#return {'warning':msgalert}
			
dos_sale_commission_report_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: