# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from osv import osv, fields
from datetime import datetime
from dateutil.relativedelta import relativedelta
from tools.translate import _

class sale_commission_report_wizard(osv.osv_memory):
	_name = 'sale.commission.report.wizard'
	_description = 'Sale Commissions Report'
	
	def _get_first_day_present_year(self, cr, uid, context=None):
		if context is None:
			context = {}
		date_ref = datetime.now().strftime('%Y-%m-%d')
		first_date = (datetime.strptime(date_ref, '%Y-%m-%d') + relativedelta(day=1, month=1))
		return first_date.strftime('%Y-%m-%d')
		# return datetime.now()
	
	_columns = {
		'date_start': fields.date('From invoice Date' ),
		'date_end': fields.date('To invoice Date' ),
		'payment_date_start': fields.date('From payment date'),
		'payment_date_end': fields.date('To payment date'),
		'settled': fields.selection([
			('all','All'),
			('not_settled','Not Settled'),
			('settled','Settled'),
			],'Settled' ),
		'final_report': fields.boolean('Definitivo'),
		'agent_id': fields.many2one('sale.agent','Sales Agent'),
	}
	
	_defaults = {
		'date_start': _get_first_day_present_year,
		'date_end': time.strftime('%Y-%m-%d'),
		'payment_date_start': _get_first_day_present_year,
		'payment_date_end': time.strftime('%Y-%m-%d'),
		'settled': 'all',
		'final_report': False,
	}
	
	def _get_agent_ids(self, cr, uid, data, context=None):
		
		inv_obj = self.pool.get('account.invoice')
		inv_ids = []
			
		if data['agent_id']:
			if data['settled'] == 'all':
				inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end']), ('agent_id', '=', data['agent_id']) ])
			else:
				settled = False
				if data['settled'] == 'settled':
					settled = True
				inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end']), ('agent_id', '=', data['agent_id'])])
		else:
			if data['settled'] == 'all':
				inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end'])])
			else:
				settled = False
				if data['settled'] == 'settled':
					settled = True
				inv_ids = inv_obj.search(cr, uid, [ ('state','=', 'paid'), ('commission_settled','=', settled), ('date_invoice','>=',data['date_start']), ('date_invoice','<=',data['date_end']), ('paid_date','>=',data['payment_date_start']), ('paid_date','<=',data['payment_date_end'])])
		res = {}
		
		invoice_ids = inv_obj.browse(cr,uid,inv_ids)
		
		for invoice in invoice_ids:
			if invoice.agent_id and not invoice.agent_id.id in res:
				res[invoice.agent_id.id] = invoice.agent_id.id
		
		return res.values()
	
	def check_report(self, cr, uid, ids, context=None):
		datas = {}
		if context is None:
			context = {}
		
		data = self.read(cr, uid, ids, [], context=context)[0]
		agent_ids = self._get_agent_ids(cr, uid, data, context=context)
		
		datas = {
			 'ids': agent_ids, #context.get('active_ids',[]),
			 'model': 'sale.agent',
			 'form': data
		}
		if agent_ids and len(agent_ids) > 0:
			return {
				'type': 'ir.actions.report.xml',
				'report_name': 'sale_commissions_report',
				'datas': datas,
			}
		else:
			raise osv.except_osv(_('Warning !'),'No hay facturas en los limites indicados.')
			#msgalert = {'title':'Warning','message':'No hay facturas en los limites indicados'}
			#return {'warning':msgalert}
			
sale_commission_report_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: