# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2012 DOS (http://www.dos-sl.es) All Rights Reserved.
#
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from mx.DateTime import *
from report import report_sxw
import xml
import rml_parse
import pooler
from tools.translate import _

class print_vat_clearance_list(rml_parse.rml_parse):
    """ Report that print invoices grouped by currency and type """
    _name = 'report.account.print_vat_clearance_list'


    def __init__(self, cr, uid, name, context):
        super(print_vat_clearance_list, self).__init__(cr, uid, name, context)
        #contain tuples of in invoices : (curreny, [browse records], currency_untaxed, currency_tax, currency_total)
        self.in_invoices = []

        #contain tuples of out invoices : (curreny, [browse records], currency_untaxed, currency_tax, currency_total)
        self.out_invoices = []

        #contain difference
        self.differences = []
        
        self.localcontext.update({
            'time': time,
            'in_invoices': self.in_invoices,
            'out_invoices': self.out_invoices,
            'differences': self.differences,
            'vat_taxes': False,
            'irpf_taxes': False,
            'report_name': '',
        })


    def set_context(self, objects, data, ids, report_type = None):
        """We do the grouping and proccessing of invoices"""
        invoice_obj = self.pool.get('account.invoice')

        if data['model'] == 'ir.ui.menu':
            invoice_types = []
            if data['form']['out_invoice']:
                invoice_types.append('out_invoice')
            if data['form']['out_refund']:
                invoice_types.append('out_refund')
            if data['form']['in_invoice']:
                invoice_types.append('in_invoice')
            if data['form']['in_refund']:
                invoice_types.append('in_refund')
            invoice_states = []
            if data['form']['draft']:
                invoice_states.append('draft')
            if data['form']['proforma']:
                invoice_states.append('proforma')
                invoice_states.append('proforma2')
            if data['form']['open']:
                invoice_states.append('open')
            if data['form']['paid']:
                invoice_states.append('paid')
            if data['form']['cancel']:
                invoice_states.append('cancel')
            where = [('company_id','=',data['form']['company_id']),('type','in',invoice_types),('state','in',invoice_states)]
            if data['form']['state'] in ['byperiod','all']:
                period_ids = data['form']['periods'][0][2]
                periods = ','.join([str(id) for id in period_ids])
                where.append(('period_id','in',period_ids))
            if data['form']['state'] in ['bydate','all','none']:
                where.append(('date_invoice','>=',data['form']['date_from']))
                where.append(('date_invoice','<=',data['form']['date_to']))
            #print where
            ids = invoice_obj.search(self.cr, self.uid, where)
            objects = invoice_obj.browse(self.cr, self.uid, ids)

            self.localcontext.update({
                'vat_taxes': data['form'].get('vat_taxes'),
                'irpf_taxes': data['form'].get('irpf_taxes'),
                'report_name': data['form'].get('report_name'),
            })
        
        vat_taxes = data['form'].get('vat_taxes') or False    
        irpf_taxes = data['form'].get('irpf_taxes') or False

        if not ids :
            return super(print_vat_clearance_list, self).set_context(objects, data, ids, report_type)
        
        if not isinstance(ids, list) :
            ids = [ids]
        # we create temp list that will be used for store invoices by type
        ininv = []
        outinv = []

        # we get the invoices and sort them by types
        invoices = invoice_obj.browse(self.cr, self.uid, ids)
        for inv in invoices :
            if inv.type in ('in_invoice', 'in_refund'):
                ininv.append(inv)
            if inv.type in ('out_invoice', 'out_refund'):
                outinv.append(inv)

        # we process the invoice and attribute them to the property
        self.filter_invoices(outinv, self.out_invoices, vat_taxes, irpf_taxes)
        self.filter_invoices(ininv, self.in_invoices, vat_taxes, irpf_taxes)
		if not irpf_taxes:
			self.calculate_differences(self.differences, self.in_invoices, self.out_invoices)
        super(print_vat_clearance_list, self).set_context(objects, data, ids, report_type)


    def filter_invoices(self, list, dest, vat_taxes=False, irpf_taxes=False):
        if not list :
            return 
        tmp = {}

        #
        # Group by currency
        #
        for inv in list:
            currency = inv.currency_id.name
            if tmp.has_key(currency) :
                tmp[currency].append(inv)
            else :
                tmp[currency] = [inv]

                
        #
        # Compute the total for each currency group
        #
        for curr in tmp:
            base = amount = total = 0
            dic_tax_lines = {}
            tax_lines = []
            
            if vat_taxes or irpf_taxes:
                
                for tmpinv in tmp[curr]:
    
                    for tax_line in tmpinv.tax_line:
                        
                        if tax_line.tax_code_id and ((tax_line.tax_code_id.irpf == irpf_taxes) or (not tax_line.tax_code_id.irpf == vat_taxes)):
                            
                            #Caso especial en caso de impuestos intracomunitarios
                            if tax_line.base_code_id.code in ('[19]') or tax_line.tax_code_id.code in ('[20]'):
                                
                                for line in self.out_invoices:
                                    added = False
                                    if line[0] == curr:
                                        for taxes in line[1]:
                                            if tax_line.name == taxes['name']:
                                                taxes['base'] = taxes['base'] + tax_line.base
                                                taxes['amount'] = taxes['amount'] - tax_line.amount
                                                taxes['total'] = taxes['total'] + (tax_line.base - tax_line.amount)
                                                line[2] = line[2] + tax_line.base
                                                line[3] = line[3] - tax_line.amount
                                                line[4] = line[4] + (tax_line.base - tax_line.amount)
                                                added = True
                                                break
                                                
                                        if not added:
                                            line[1].append({
                                                    'name': tax_line.name,
                                                    'base': tax_line.base,
                                                    'amount': -tax_line.amount,
                                                    'total': tax_line.base - tax_line.amount
                                                   })
                                            line[2] = line[2] + tax_line.base
                                            line[3] = line[3] - tax_line.amount
                                            line[4] = line[4] + (tax_line.base - tax_line.amount)
                                            added = True
                                        break
                                    
                                if not added:
                                    self.out_invoices.append([curr, [{
                                                    'name': tax_line.name,
                                                    'base': tax_line.base,
                                                    'amount': -tax_line.amount,
                                                    'total': tax_line.base - tax_line.amount                                                        
                                            }], tax_line.base, -tax_line.amount, (tax_line.base - tax_line.amount)])
                                    
                                continue
                            
                            #Caso normal
                            if tax_line.name in dic_tax_lines:
                                dic_tax_lines.update({
                                    tax_line.name: {
                                                    'name': tax_line.name,
                                                    'base': dic_tax_lines[tax_line.name]['base'] + tax_line.base,
                                                    'amount': dic_tax_lines[tax_line.name]['amount'] + tax_line.amount,
                                                    'total': dic_tax_lines[tax_line.name]['total'] + (tax_line.base + tax_line.amount)
                                                   }                                                         
                                })
                            else:
                                dic_tax_lines.update({
                                    tax_line.name: {
                                                    'name': tax_line.name,
                                                    'base': tax_line.base,
                                                    'amount': tax_line.amount,
                                                    'total': tax_line.base + tax_line.amount
                                                   }                                                         
                                })
                                
                            base += tax_line.base
                            amount += tax_line.amount
                            total += tax_line.base + tax_line.amount
                            
    
                for tax in dic_tax_lines:
                    tax_lines.append(dic_tax_lines[tax])
           
            # Append the tuple to the property
            dest.append([curr, tax_lines, base, amount, total])
        del tmp
        
        
    def calculate_differences(self, dest, in_invoices, out_invoices):
        
        dic_lines = {}
        
        #
        # Compute the total for each currency group
        #     
        for line_out in out_invoices:
            if line_out[0] in dic_lines:
                dic_lines.update({
                    line_out[0]: {
                                    'currency': line_out[0],
                                    'base': dic_lines[line_out[0]]['base'] + line_out[2],
                                    'amount': dic_lines[line_out[0]]['amount'] + line_out[3],
                                    'total': dic_lines[line_out[0]]['total'] + line_out[4],
                                    'text': (dic_lines[line_out[0]]['total'] + line_out[4]) >= 0 and _('TO ENTERED') or _('TO RETURNED'),
                                   }                                                         
                })
            else:
                dic_lines.update({
                    line_out[0]: {
                                    'currency': line_out[0],
                                    'base': line_out[2],
                                    'amount': line_out[3],
                                    'total': line_out[4],
                                    'text': line_out[4] >= 0 and _('TO ENTERED') or _('TO RETURNED'),
                                   }                                                         
                })
                
        for line_in in in_invoices:
            if line_in[0] in dic_lines:
                dic_lines.update({
                    line_in[0]: {
                                    'currency': line_in[0],
                                    'base': dic_lines[line_in[0]]['base'] - line_in[2],
                                    'amount': dic_lines[line_in[0]]['amount'] - line_in[3],
                                    'total': dic_lines[line_in[0]]['total'] - line_in[4],
                                    'text': (dic_lines[line_in[0]]['total'] - line_in[4]) >= 0 and _('TO ENTERED') or _('TO RETURNED'),
                                   }                                                         
                })
            else:
                dic_lines.update({
                    line_in[0]: {
                                    'currency': line_in[0],
                                    'base': -line_in[2],
                                    'amount': -line_in[3],
                                    'total': -line_in[4],
                                    'text': -line_in[4] >= 0 and _('TO ENTERED') or _('TO RETURNED'),
                                   }                                                     
                })
            
        #Return total difference
        for line in dic_lines:
            # Append the tuple to the property
            dest.append(dic_lines[line])


report_sxw.report_sxw('report.account.vat.clearance.list.report', 'account.invoice', 'addons/dos_vat_clearance_report/report/vat_clearance_list.rml', parser=print_vat_clearance_list, header=False)
